#define MAX_LOG_ENTRIES         100

// Write a new log entry to the ship's log
// logTitle: Title of the log entry
// logEntry: Text of the log entry
// logCategory: "General", "Battle", "Ship", "Personal", "Notes"
// bVisible: if log entry is visible (e.g. the news is known to the character) 
void WriteNewLogEntry(string logTitle, string logEntry, string logCategory, bool bVisible)
{
	ref pchar = GetMainCharacter();
	bool bUpdate;
	int i;
	int maxVal;
	string tmpLogTitle;
	string oldLog;
	string newLog;

	bUpdate = false;
	if(logCategory == GetAttribute(pchar, "shiplog.Category.log0")) // Update existing log entries to avoid log flooding
	{
		switch(logCategory)
		{
		case "Battle":
			if(GetStringDate(GetDataYear(), GetDataMonth(), GetDataDay()) == pchar.shiplog.Date.log0)
			{
				newLog = "";
				if(pchar.shiplog.Title.log0 != GetTranslatedLog("Several battles at sea")) { newLog = GetTranslatedLog("Encountered several enemy ships today"); newLog += "\n \n"; }
				pchar.shiplog.Title.log0 = GetTranslatedLog("Several battles at sea");
				newLog += pchar.shiplog.Entry.log0;
				newLog += "\n \n";
				newLog += logEntry;
				pchar.shiplog.Entry.log0 = newLog;
				bUpdate = true;
			}
			break;
		case "Ship":
			newLog = "";
			if(pchar.shiplog.Title.log0 == logTitle)
			{
				newLog += pchar.shiplog.Entry.log0;
				newLog += "\n \n";
				if(GetStringDate(GetDataYear(), GetDataMonth(), GetDataDay()) != pchar.shiplog.Date.log0) newLog += GetDayName(GetWeekday(GetDataDay(), GetDataMonth(), GetDataYear()))+", "+GetHumanDate(GetDataYear(), GetDataMonth(), GetDataDay())+": ";
				newLog += logEntry;
				pchar.shiplog.Entry.log0 = newLog;
				bUpdate = true;
			}
			break;
		}
	}

	if(bUpdate) return;
	// move all log entries one down.
	aref olddata, newdata;
	for(i=MAX_LOG_ENTRIES-1;i>=0;i--)
	{
		newLog = "log"+i;
		oldLog = "log"+(i+1);
		if(CheckAttribute(pchar, "shiplog.Title."+newLog))    
		{
			pchar.shiplog.Date.(oldLog) = pchar.shiplog.Date.(newLog);            
			pchar.shiplog.Title.(oldLog) = pchar.shiplog.Title.(newLog);
			pchar.shiplog.Entry.(oldLog) = pchar.shiplog.Entry.(newLog);
			pchar.shiplog.Category.(oldLog) = pchar.shiplog.Category.(newLog);
			pchar.shiplog.Visible.(oldLog) = pchar.shiplog.Visible.(newLog);
			pchar.shiplog.Event.(oldLog) = pchar.shiplog.Event.(newLog); //Levis: added for log events on becoming visible
			//Also take the data
			makearef(olddata, pchar.shiplog.Event.(oldLog).data);
			makearef(newdata, pchar.shiplog.Event.(newLog).data);
			CopyAttributes(olddata, newdata);
			//Clear the old data
			DeleteAttribute(pchar,"shiplog.Event."+newlog+".data");
		}
	} 
	// set the new log entry.
	pchar.shiplog.Date.log0 = GetStringDate(GetDataYear(), GetDataMonth(), GetDataDay());
	pchar.shiplog.Title.log0 = logTitle;
	pchar.shiplog.Entry.log0 = logEntry;
	pchar.shiplog.Category.log0 = logCategory;
	pchar.shiplog.Visible.log0 = bVisible;
	pchar.shiplog.Event.log0 = ""; //Levis

	// PB: Sound and on-screen effects -->
	if (bVisible)
	{
		if (logCategory == "General" || logCategory == "Personal")
		{
			Log_SetStringToLog(XI_ConvertString("Ship Log Update"));
			AddMsgToCharacter(pchar,MSGICON_LOGBOOK);
			PlaySound("INTERFACE\notebook_01.wav");
		}
	}
	// PB: Sound and on-screen effects <--
}

//Levis: Add option to call functions when log becomes visible (so when the player has seen it)
//You can select a function to call and you can send a reference to a dataset which will be added and send to the function. In here you can store data.
void SetLogEvent(string logTitle, string functionname, ref data)
{
	ref pchar = GetMainCharacter();
	
	//First find the log
	string logid = FindLogID(logTitle);
	
	//Add the data to the entry but only if not visible
	if(pchar.shiplog.Visible.(logid))
	{
		//The log is already visible so we send an error
		trace("LOG ERROR: set event for "+logTitle+" but log is already visible.");
	}
	else
	{
		//Set the event
		pchar.shiplog.Event.(logid) = functionname;
		//Check if we have an empty data object
		if(GetAttributesNum(data) <= 0)
		{
			data.value = "";
		}
		//Store all data attributes
		aref eventdata; makearef(eventdata, pchar.shiplog.Event.(logid).data);
		CopyAttributes(eventdata, data);
	}
}

void CallLogEvent(string logid)
{
	//This function calls the event stored in the log entry
	ref pchar = GetMainCharacter();
	
	//Get the function name
	string func = pchar.shiplog.Event.(logid);
	
	if(func != "")
	{
		//Get the data
		aref data; makearef(data, pchar.shiplog.Event.(logid).data);
		
		//call the function
		trace("LOG EVENT: called function "+func+" due to logentry "+logid);
		call func(data);
		
		//set as called
		pchar.shiplog.Event.(logid).called = true;
	}
	else
	{
		//This function was called for an empty event which shouldn't happen.
		trace("LOG ERROR: empty event called for log entry: "+logid);
	}
}

string FindLogID(string logTitle)
{
	//This function returns the log id (like: log12) as a string. You can search by the logTitle
	string logid;
	ref pchar = GetMainCharacter();
	for(int i=0;i<=MAX_LOG_ENTRIES;i++)
	{
		logid = "log"+i;
		//Check if the attribute exists, if not we reached the end of the log messages
		if(CheckAttribute(pchar,"shiplog.Title."+logid))
		{
			if(pchar.shiplog.Title.(logid) == logTitle)
			{
				return logid;
			}
		}
		else
		{
			break;
		}
	}
	//return an empty value if nothing is found
	return "";
}

// Make all invisible log entries visible (e.g. after reading news at tavern)
void SetLogVisible()
{
	int i;
	string tmp;
	
	for(i=0;i<=MAX_LOG_ENTRIES;i++)
	{
		tmp = "log"+i;
		SetSingleLogVisible(tmp)
	}
}

// Levis: Seperating stuff in different functions so later we can use it better

void SetSingleLogVisible(string title)
{
	ref pchar = GetMainCharacter();
	if(CheckAttribute(pchar, "shiplog.Title."+title))    
	{
		pchar.shiplog.Visible.(title) = true;
		
		//Check if there is an event
		if(pchar.shiplog.Event.(title) != "")
		{
			if(!CheckAttribute(pchar,"shiplog.Event."+title+".called"))
			{
				CallLogEvent(title);
			}
		}
	}
}

// Check, if log entry with such title already exists
bool LogEntryExists(string logTitle)
{
	ref pchar = GetMainCharacter();
	int i;
	string tmp;
	
	for(i=0;i<=MAX_LOG_ENTRIES;i++)
	{
		tmp = "log"+i;
		if(CheckAttribute(pchar, "shiplog.Title."+tmp))    
		{
			if(pchar.shiplog.Title.(tmp) == logTitle)
			{
				return true;
			}        
		}
	}
	return false;
}

void UpdateStatistics(string sStat, int num)
{
	ref pchar = GetMainCharacter();
	pchar.shiplog.Stats.(sStat) = sti(pchar.shiplog.Stats.(sStat))+num;
}

string GetStatistics(string sStat)
{
	ref pchar = GetMainCharacter();
	if(CheckAttribute(pchar, "shiplog.Stats."+sStat)) return pchar.shiplog.Stats.(sStat);
	else return "0";
}


string GetLogTime()
{
	int i = makeint(GetTime());
	string tmp;
	string sDateTime;  

	tmp="around midnight";
	
	switch(i)
	{
	case 2:
		tmp="in the night, quite some time after midnight";
		break;
	case 3:
		tmp="in the night, quite some time after midnight";
		break;
	case 4:
		tmp="in the early morning, before dawn,";
		break;
	case 5:
		tmp="in the early morning, before dawn,";
		break;
	case 6:
		tmp="in the early morning, with the sun rising,";
		break;
	case 7:
		tmp="in the early morning, with the sun rising,";
		break;
	case 8:
		tmp="in the morning";
		break;
	case 9:
		tmp="in the morning";
		break;
	case 10:
		tmp="in the morning";
		break;
	case 11:
		tmp="around noon";
		break;
	case 12:
		tmp="around noon";
		break;
	case 13:
		tmp="around noon";
		break;
	case 14:
		tmp="in the early afternoon";
		break;
	case 15:
		tmp="in the early afternoon";
		break;
	case 16:
		tmp="in the afternoon";
		break;
	case 17:
		tmp="in the afternoon";
		break;
	case 18:
		tmp="in the evening";
		break;
	case 19:
		tmp="in the evening";
		break;
	case 20:
		tmp="in the evening, the sun was not yet set,";
		break;
	case 21:
		tmp="in the evening, the sun was not yet set,";
		break;
	case 22:
		tmp="in the late evening after sunset";
		break;
	case 23: 
		tmp="in the night";   
		break;
	}

	sDateTime=GetTranslatedLog("On ")+GetDayName(GetWeekday(GetDataDay(), GetDataMonth(), GetDataYear()))+", "+GetHumanDate(GetDataYear(), GetDataMonth(), GetDataDay())+" "+GetTranslatedLog(tmp);
	return sDateTime;
}
